# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'plotConfigTemplate.ui'
##
## Created by: Qt User Interface Compiler version 6.1.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(481, 840)
        self.averageGroup = QGroupBox(Form)
        self.averageGroup.setObjectName(u"averageGroup")
        self.averageGroup.setGeometry(QRect(0, 640, 242, 182))
        self.averageGroup.setCheckable(True)
        self.averageGroup.setChecked(False)
        self.gridLayout_5 = QGridLayout(self.averageGroup)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.avgParamList = QListWidget(self.averageGroup)
        self.avgParamList.setObjectName(u"avgParamList")

        self.gridLayout_5.addWidget(self.avgParamList, 0, 0, 1, 1)

        self.decimateGroup = QFrame(Form)
        self.decimateGroup.setObjectName(u"decimateGroup")
        self.decimateGroup.setGeometry(QRect(10, 140, 191, 171))
        self.gridLayout_4 = QGridLayout(self.decimateGroup)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.clipToViewCheck = QCheckBox(self.decimateGroup)
        self.clipToViewCheck.setObjectName(u"clipToViewCheck")

        self.gridLayout_4.addWidget(self.clipToViewCheck, 7, 0, 1, 3)

        self.maxTracesCheck = QCheckBox(self.decimateGroup)
        self.maxTracesCheck.setObjectName(u"maxTracesCheck")

        self.gridLayout_4.addWidget(self.maxTracesCheck, 8, 0, 1, 2)

        self.downsampleCheck = QCheckBox(self.decimateGroup)
        self.downsampleCheck.setObjectName(u"downsampleCheck")

        self.gridLayout_4.addWidget(self.downsampleCheck, 0, 0, 1, 3)

        self.peakRadio = QRadioButton(self.decimateGroup)
        self.peakRadio.setObjectName(u"peakRadio")
        self.peakRadio.setChecked(True)

        self.gridLayout_4.addWidget(self.peakRadio, 6, 1, 1, 2)

        self.maxTracesSpin = QSpinBox(self.decimateGroup)
        self.maxTracesSpin.setObjectName(u"maxTracesSpin")

        self.gridLayout_4.addWidget(self.maxTracesSpin, 8, 2, 1, 1)

        self.forgetTracesCheck = QCheckBox(self.decimateGroup)
        self.forgetTracesCheck.setObjectName(u"forgetTracesCheck")

        self.gridLayout_4.addWidget(self.forgetTracesCheck, 9, 0, 1, 3)

        self.meanRadio = QRadioButton(self.decimateGroup)
        self.meanRadio.setObjectName(u"meanRadio")

        self.gridLayout_4.addWidget(self.meanRadio, 3, 1, 1, 2)

        self.subsampleRadio = QRadioButton(self.decimateGroup)
        self.subsampleRadio.setObjectName(u"subsampleRadio")

        self.gridLayout_4.addWidget(self.subsampleRadio, 2, 1, 1, 2)

        self.autoDownsampleCheck = QCheckBox(self.decimateGroup)
        self.autoDownsampleCheck.setObjectName(u"autoDownsampleCheck")
        self.autoDownsampleCheck.setChecked(True)

        self.gridLayout_4.addWidget(self.autoDownsampleCheck, 1, 2, 1, 1)

        self.horizontalSpacer = QSpacerItem(30, 20, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.gridLayout_4.addItem(self.horizontalSpacer, 2, 0, 1, 1)

        self.downsampleSpin = QSpinBox(self.decimateGroup)
        self.downsampleSpin.setObjectName(u"downsampleSpin")
        self.downsampleSpin.setMinimum(1)
        self.downsampleSpin.setMaximum(100000)
        self.downsampleSpin.setValue(1)

        self.gridLayout_4.addWidget(self.downsampleSpin, 1, 1, 1, 1)

        self.transformGroup = QFrame(Form)
        self.transformGroup.setObjectName(u"transformGroup")
        self.transformGroup.setGeometry(QRect(10, 10, 171, 101))
        self.gridLayout = QGridLayout(self.transformGroup)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName(u"gridLayout")
        self.logYCheck = QCheckBox(self.transformGroup)
        self.logYCheck.setObjectName(u"logYCheck")

        self.gridLayout.addWidget(self.logYCheck, 2, 0, 1, 1)

        self.logXCheck = QCheckBox(self.transformGroup)
        self.logXCheck.setObjectName(u"logXCheck")

        self.gridLayout.addWidget(self.logXCheck, 1, 0, 1, 1)

        self.fftCheck = QCheckBox(self.transformGroup)
        self.fftCheck.setObjectName(u"fftCheck")

        self.gridLayout.addWidget(self.fftCheck, 0, 0, 1, 1)

        self.derivativeCheck = QCheckBox(self.transformGroup)
        self.derivativeCheck.setObjectName(u"derivativeCheck")

        self.gridLayout.addWidget(self.derivativeCheck, 3, 0, 1, 1)

        self.phasemapCheck = QCheckBox(self.transformGroup)
        self.phasemapCheck.setObjectName(u"phasemapCheck")

        self.gridLayout.addWidget(self.phasemapCheck, 4, 0, 1, 1)

        self.pointsGroup = QGroupBox(Form)
        self.pointsGroup.setObjectName(u"pointsGroup")
        self.pointsGroup.setGeometry(QRect(10, 550, 234, 58))
        self.pointsGroup.setCheckable(True)
        self.verticalLayout_5 = QVBoxLayout(self.pointsGroup)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.autoPointsCheck = QCheckBox(self.pointsGroup)
        self.autoPointsCheck.setObjectName(u"autoPointsCheck")
        self.autoPointsCheck.setChecked(True)

        self.verticalLayout_5.addWidget(self.autoPointsCheck)

        self.gridGroup = QFrame(Form)
        self.gridGroup.setObjectName(u"gridGroup")
        self.gridGroup.setGeometry(QRect(10, 460, 221, 81))
        self.gridLayout_2 = QGridLayout(self.gridGroup)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.xGridCheck = QCheckBox(self.gridGroup)
        self.xGridCheck.setObjectName(u"xGridCheck")

        self.gridLayout_2.addWidget(self.xGridCheck, 0, 0, 1, 2)

        self.yGridCheck = QCheckBox(self.gridGroup)
        self.yGridCheck.setObjectName(u"yGridCheck")

        self.gridLayout_2.addWidget(self.yGridCheck, 1, 0, 1, 2)

        self.gridAlphaSlider = QSlider(self.gridGroup)
        self.gridAlphaSlider.setObjectName(u"gridAlphaSlider")
        self.gridAlphaSlider.setMaximum(255)
        self.gridAlphaSlider.setValue(128)
        self.gridAlphaSlider.setOrientation(Qt.Horizontal)

        self.gridLayout_2.addWidget(self.gridAlphaSlider, 2, 1, 1, 1)

        self.label = QLabel(self.gridGroup)
        self.label.setObjectName(u"label")

        self.gridLayout_2.addWidget(self.label, 2, 0, 1, 1)

        self.alphaGroup = QGroupBox(Form)
        self.alphaGroup.setObjectName(u"alphaGroup")
        self.alphaGroup.setGeometry(QRect(10, 390, 234, 60))
        self.alphaGroup.setCheckable(True)
        self.horizontalLayout = QHBoxLayout(self.alphaGroup)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.autoAlphaCheck = QCheckBox(self.alphaGroup)
        self.autoAlphaCheck.setObjectName(u"autoAlphaCheck")
        self.autoAlphaCheck.setChecked(False)

        self.horizontalLayout.addWidget(self.autoAlphaCheck)

        self.alphaSlider = QSlider(self.alphaGroup)
        self.alphaSlider.setObjectName(u"alphaSlider")
        self.alphaSlider.setMaximum(1000)
        self.alphaSlider.setValue(1000)
        self.alphaSlider.setOrientation(Qt.Horizontal)

        self.horizontalLayout.addWidget(self.alphaSlider)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"PyQtGraph", None))
#if QT_CONFIG(tooltip)
        self.averageGroup.setToolTip(QCoreApplication.translate("Form", u"Display averages of the curves displayed in this plot. The parameter list allows you to choose parameters to average over (if any are available).", None))
#endif // QT_CONFIG(tooltip)
        self.averageGroup.setTitle(QCoreApplication.translate("Form", u"Average", None))
#if QT_CONFIG(tooltip)
        self.clipToViewCheck.setToolTip(QCoreApplication.translate("Form", u"Plot only the portion of each curve that is visible. This assumes X values are uniformly spaced.", None))
#endif // QT_CONFIG(tooltip)
        self.clipToViewCheck.setText(QCoreApplication.translate("Form", u"Clip to View", None))
#if QT_CONFIG(tooltip)
        self.maxTracesCheck.setToolTip(QCoreApplication.translate("Form", u"If multiple curves are displayed in this plot, check this box to limit the number of traces that are displayed.", None))
#endif // QT_CONFIG(tooltip)
        self.maxTracesCheck.setText(QCoreApplication.translate("Form", u"Max Traces:", None))
        self.downsampleCheck.setText(QCoreApplication.translate("Form", u"Downsample", None))
#if QT_CONFIG(tooltip)
        self.peakRadio.setToolTip(QCoreApplication.translate("Form", u"Downsample by drawing a saw wave that follows the min and max of the original data. This method produces the best visual representation of the data but is slower.", None))
#endif // QT_CONFIG(tooltip)
        self.peakRadio.setText(QCoreApplication.translate("Form", u"Peak", None))
#if QT_CONFIG(tooltip)
        self.maxTracesSpin.setToolTip(QCoreApplication.translate("Form", u"If multiple curves are displayed in this plot, check \"Max Traces\" and set this value to limit the number of traces that are displayed.", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.forgetTracesCheck.setToolTip(QCoreApplication.translate("Form", u"If MaxTraces is checked, remove curves from memory after they are hidden (saves memory, but traces can not be un-hidden).", None))
#endif // QT_CONFIG(tooltip)
        self.forgetTracesCheck.setText(QCoreApplication.translate("Form", u"Forget hidden traces", None))
#if QT_CONFIG(tooltip)
        self.meanRadio.setToolTip(QCoreApplication.translate("Form", u"Downsample by taking the mean of N samples.", None))
#endif // QT_CONFIG(tooltip)
        self.meanRadio.setText(QCoreApplication.translate("Form", u"Mean", None))
#if QT_CONFIG(tooltip)
        self.subsampleRadio.setToolTip(QCoreApplication.translate("Form", u"Downsample by taking the first of N samples. This method is fastest and least accurate.", None))
#endif // QT_CONFIG(tooltip)
        self.subsampleRadio.setText(QCoreApplication.translate("Form", u"Subsample", None))
#if QT_CONFIG(tooltip)
        self.autoDownsampleCheck.setToolTip(QCoreApplication.translate("Form", u"Automatically downsample data based on the visible range. This assumes X values are uniformly spaced.", None))
#endif // QT_CONFIG(tooltip)
        self.autoDownsampleCheck.setText(QCoreApplication.translate("Form", u"Auto", None))
#if QT_CONFIG(tooltip)
        self.downsampleSpin.setToolTip(QCoreApplication.translate("Form", u"Downsample data before plotting. (plot every Nth sample)", None))
#endif // QT_CONFIG(tooltip)
        self.downsampleSpin.setSuffix(QCoreApplication.translate("Form", u"x", None))
        self.logYCheck.setText(QCoreApplication.translate("Form", u"Log Y", None))
        self.logXCheck.setText(QCoreApplication.translate("Form", u"Log X", None))
        self.fftCheck.setText(QCoreApplication.translate("Form", u"Power Spectrum (FFT)", None))
        self.derivativeCheck.setText(QCoreApplication.translate("Form", u"dy/dx", None))
        self.phasemapCheck.setText(QCoreApplication.translate("Form", u"Y vs. Y'", None))
        self.pointsGroup.setTitle(QCoreApplication.translate("Form", u"Points", None))
        self.autoPointsCheck.setText(QCoreApplication.translate("Form", u"Auto", None))
        self.xGridCheck.setText(QCoreApplication.translate("Form", u"Show X Grid", None))
        self.yGridCheck.setText(QCoreApplication.translate("Form", u"Show Y Grid", None))
        self.label.setText(QCoreApplication.translate("Form", u"Opacity", None))
        self.alphaGroup.setTitle(QCoreApplication.translate("Form", u"Alpha", None))
        self.autoAlphaCheck.setText(QCoreApplication.translate("Form", u"Auto", None))
    # retranslateUi




# -*- coding: utf-8 -*-
"""
ViewBox is the general-purpose graphical container that allows the user to 
zoom / pan to inspect any area of a 2D coordinate system. 

This example demonstrates many of the features ViewBox provides.
"""

import initExample ## Add path to library (just for examples; you do not need this)

import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui
import numpy as np

x = np.arange(1000, dtype=float)
y = np.random.normal(size=1000)
y += 5 * np.sin(x/100) 

win = pg.GraphicsLayoutWidget(show=True)
win.setWindowTitle('pyqtgraph example: ____')
win.resize(1000, 800)
win.ci.setBorder((50, 50, 100))

sub1 = win.addLayout()
sub1.addLabel("<b>Standard mouse interaction:</b><br>left-drag to pan, right-drag to zoom.")
sub1.nextRow()
v1 = sub1.addViewBox()
l1 = pg.PlotDataItem(y)
v1.addItem(l1)


sub2 = win.addLayout()
sub2.addLabel("<b>One-button mouse interaction:</b><br>left-drag zoom to box, wheel to zoom out.")
sub2.nextRow()
v2 = sub2.addViewBox()
v2.setMouseMode(v2.RectMode)
l2 = pg.PlotDataItem(y)
v2.addItem(l2)

win.nextRow()

sub3 = win.addLayout()
sub3.addLabel("<b>Locked aspect ratio when zooming.</b>")
sub3.nextRow()
v3 = sub3.addViewBox()
v3.setAspectLocked(1.0)
l3 = pg.PlotDataItem(y)
v3.addItem(l3)

sub4 = win.addLayout()
sub4.addLabel("<b>View limits:</b><br>prevent panning or zooming past limits.")
sub4.nextRow()
v4 = sub4.addViewBox()
v4.setLimits(xMin=-100, xMax=1100, 
             minXRange=20, maxXRange=500, 
             yMin=-10, yMax=10,
             minYRange=1, maxYRange=10)
l4 = pg.PlotDataItem(y)
v4.addItem(l4)

win.nextRow()

sub5 = win.addLayout()
sub5.addLabel("<b>Linked axes:</b> Data in this plot is always X-aligned to<br>the plot above.")
sub5.nextRow()
v5 = sub5.addViewBox()
v5.setXLink(v3)
l5 = pg.PlotDataItem(y)
v5.addItem(l5)

sub6 = win.addLayout()
sub6.addLabel("<b>Disable mouse:</b> Per-axis control over mouse input.<br>"
              "<b>Auto-scale-visible:</b> Automatically fit *visible* data within view<br>"
              "(try panning left-right).")
sub6.nextRow()
v6 = sub6.addViewBox()
v6.setMouseEnabled(x=True, y=False)
v6.enableAutoRange(x=False, y=True)
v6.setXRange(300, 450)
v6.setAutoVisible(x=False, y=True)
l6 = pg.PlotDataItem(y)
v6.addItem(l6)

if __name__ == '__main__':
    pg.exec()



# -*- coding: utf-8 -*-
from ..Qt import QtCore
from .. import functions as fn
from .PlotItem import PlotItem
from .ImageItem import ImageItem
from .LinearRegionItem import LinearRegionItem

import weakref
import math
import warnings
import numpy as np

__all__ = ['ColorBarItem']

class ColorBarItem(PlotItem):
    """
    **Bases:** :class:`PlotItem <pyqtgraph.PlotItem>`

    ColorBarItem is a simpler, compact alternative to HistogramLUTItem, without histogram
    or the option to adjust the look-up table.

    A labeled axis is displayed directly next to the gradient to help identify values.
    Handles included in the color bar allow for interactive adjustment.

    A ColorBarItem can be assigned one or more ImageItems that will be displayed according
    to the selected color map and levels. The ColorBarItem can be used as a separate
    element in a GraphicsLayout or added to the layout of a PlotItem used to display image
    data with coordinate axes.

    =============================  =============================================
    **Signals:**
    sigLevelsChanged(self)         Emitted when the range sliders are moved
    sigLevelsChangeFinished(self)  Emitted when the range sliders are released
    =============================  =============================================
    """
    sigLevelsChanged = QtCore.Signal(object)
    sigLevelsChangeFinished = QtCore.Signal(object)

    def __init__(self, values=(0,1), width=25, colorMap=None, label=None,
                 interactive=True, limits=None, rounding=1,
                 orientation='vertical', pen='w', hoverPen='r', hoverBrush='#FF000080', cmap=None ):
        """
        Creates a new ColorBarItem.

        ==============  ===========================================================================
        **Arguments:**
        values          The range of values as tuple (min, max)
        width           (default=25) The width of the displayed color bar
        colorMap        ColorMap object, look-up table is also applied to assigned ImageItem(s)
        label           (default=None) Label applied to color bar axis
        interactive     (default=True) Handles are displayed to interactively adjust level range
        limits          Limits to adjustment range as (low, high) tuple, None disables limit
        rounding        (default=1) Adjusted range values are rounded to multiples of this values
        orientation     (default='vertical') 'horizontal' or 'h' gives a horizontal color bar.
        pen             color of adjustement handles in interactive mode
        hoverPen        color of adjustement handles when hovered over
        hoverBrush      color of movable center region when hovered over
        ==============  ===========================================================================
        """
        super().__init__()
        if cmap is not None: 
            warnings.warn(
                "The parameter 'cmap' has been renamed to 'colorMap' for clarity. "
                "The old name will no longer be available in any version of PyQtGraph released after July 2022.",
                DeprecationWarning, stacklevel=2
            )
            colorMap = cmap
        self.img_list  = [] # list of controlled ImageItems
        self.values    = values
        self._colorMap = colorMap
        self.rounding  = rounding
        self.horizontal = bool( orientation in ('h', 'horizontal') )

        self.lo_prv, self.hi_prv = self.values # remember previous values while adjusting range
        if limits is None:
            self.lo_lim = None
            self.hi_lim = None
        else:
            self.lo_lim, self.hi_lim = limits

        self.disableAutoRange()
        self.hideButtons()
        self.setMouseEnabled(x=False, y=False)
        self.setMenuEnabled( False)

        if self.horizontal:
            self.setRange( xRange=(0,256), yRange=(0,1), padding=0 )
            self.layout.setRowFixedHeight(2, width)
        else:
            self.setRange( xRange=(0,1), yRange=(0,256), padding=0 )
            self.layout.setColumnFixedWidth(1, width) # width of color bar

        for key in ['left','right','top','bottom']:
            self.showAxis(key)
            axis = self.getAxis(key)
            axis.setZValue(0.5)
            # select main axis:
            if self.horizontal and key == 'bottom':
                self.axis = axis
            elif not self.horizontal and key == 'right':
                self.axis = axis
                self.axis.setWidth(45)
            else: # show other axes to create frame
                axis.setTicks( [] )
                axis.setStyle( showValues=False )
        self.axis.setStyle( showValues=True )
        self.axis.unlinkFromView()
        self.axis.setRange( self.values[0], self.values[1] )

        self.bar = ImageItem(axisOrder='col-major')
        if self.horizontal:
            self.bar.setImage( np.linspace(0, 1, 256).reshape( (-1,1) ) )
            if label is not None: self.getAxis('bottom').setLabel(label)
        else:
            self.bar.setImage( np.linspace(0, 1, 256).reshape( (1,-1) ) )
            if label is not None: self.getAxis('left').setLabel(label)
        self.addItem(self.bar)
        if cmap is not None: self.setColorMap(cmap)

        if interactive:
            if self.horizontal:
                align = 'vertical'
            else:
                align = 'horizontal'
            self.region = LinearRegionItem(
                [63, 191], align, swapMode='block',
                # span=(0.15, 0.85),  # limited span looks better, but disables grabbing the region
                pen=pen, brush=fn.mkBrush(None), hoverPen=hoverPen, hoverBrush=hoverBrush )
            self.region.setZValue(1000)
            self.region.lines[0].addMarker('<|>', size=6)
            self.region.lines[1].addMarker('<|>', size=6)
            self.region.sigRegionChanged.connect(self._regionChanging)
            self.region.sigRegionChangeFinished.connect(self._regionChanged)
            self.addItem(self.region)
            self.region_changed_enable = True
            self.region.setRegion( (63, 191) ) # place handles at 25% and 75% locations
        else:
            self.region = None
            self.region_changed_enable = False

    def setImageItem(self, img, insert_in=None):
        """
        assign ImageItem or list of ImageItems to be controlled

        ==============  ==========================================================================
        **Arguments:**
        image           ImageItem or list of [ImageItem, ImageItem, ...] that will be set to the
                        color map of the ColorBarItem. In interactive mode, the levels of all
                        assigned ImageItems will be controlled simultaneously.
        insert_in       If a PlotItem is given, the color bar is inserted on the right or bottom
                        of the plot
        ==============  ==========================================================================
        """
        try:
            self.img_list = [ weakref.ref(item) for item in img ]
        except TypeError: # failed to iterate, make a single-item list
            self.img_list = [ weakref.ref( img ) ]
        if insert_in is not None:
            if self.horizontal:
                insert_in.layout.addItem( self, 5, 1 ) # insert in layout below bottom axis
                insert_in.layout.setRowFixedHeight(4, 10) # enforce some space to axis above
            else:
                insert_in.layout.addItem( self, 2, 5 ) # insert in layout after right-hand axis
                insert_in.layout.setColumnFixedWidth(4, 5) # enforce some space to axis on the left
        self._update_items( update_cmap = True )

    # Maintain compatibility for old name of color bar setting method.
    def setCmap(self, cmap):
        warnings.warn(
            "The method 'setCmap' has been renamed to 'setColorMap' for clarity. "
            "The old name will no longer be available in any version of PyQtGraph released after July 2022.",
            DeprecationWarning, stacklevel=2
        )
        self.setColorMap(cmap)

    def setColorMap(self, colorMap):
        """
        Sets a ColorMap object to determine the ColorBarItem's look-up table. The same
        look-up table is applied to any assigned ImageItem.
        """
        self._colorMap = colorMap
        self._update_items( update_cmap = True )
        
    def colorMap(self):
        """
        Returns the assigned ColorMap object.
        """
        return self._colorMap
        
    @property
    def cmap(self):
        warnings.warn(
            "Direct access to ColorMap.cmap is deprecated and will no longer be available in any "
            "version of PyQtGraph released after July 2022. Please use 'ColorMap.colorMap()' instead.",
            DeprecationWarning, stacklevel=2)
        return self._colorMap

    def setLevels(self, values=None, low=None, high=None ):
        """
        Sets the displayed range of levels as specified.

        ==============  ===========================================================================
        **Arguments:**
        values          specifies levels as tuple (low, high). Either value can be None to leave
                        to previous value unchanged. Takes precedence over low and high parameters.
        low             new low level to be applied to color bar and assigned images
        high            new high level to be applied to color bar and assigned images
        ==============  ===========================================================================
        """
        if values is not None: # values setting takes precendence
            low, high = values
        lo_new, hi_new = low, high
        lo_cur, hi_cur = self.values
        # allow None values to preserve original values:
        if lo_new is None: lo_new = lo_cur
        if hi_new is None: hi_new = hi_cur
        if lo_new > hi_new: # prevent reversal
            lo_new = hi_new = (lo_new + hi_new) / 2
        # clip to limits if set:
        if self.lo_lim is not None and lo_new < self.lo_lim: lo_new = self.lo_lim
        if self.hi_lim is not None and hi_new > self.hi_lim: hi_new = self.hi_lim
        self.values = self.lo_prv, self.hi_prv = (lo_new, hi_new)
        self._update_items()

    def levels(self):
        """ returns the currently set levels as the tuple (low, high). """
        return self.values

    def _update_items(self, update_cmap=False):
        """ internal: update color maps for bar and assigned ImageItems """
        # update color bar:
        self.axis.setRange( self.values[0], self.values[1] )
        if update_cmap and self._colorMap is not None:
            self.bar.setLookupTable( self._colorMap.getLookupTable(nPts=256) )
        # update assigned ImageItems, too:
        for img_weakref in self.img_list:
            img = img_weakref()
            if img is None: continue # dereference weakref
            img.setLevels( self.values ) # (min,max) tuple
            if update_cmap and self._colorMap is not None:
                img.setLookupTable( self._colorMap.getLookupTable(nPts=256) )

    def _regionChanged(self):
        """ internal: snap adjusters back to default positions on release """
        self.lo_prv, self.hi_prv = self.values
        self.region_changed_enable = False # otherwise this affects the region again
        self.region.setRegion( (63, 191) )
        self.region_changed_enable = True
        self.sigLevelsChangeFinished.emit(self)

    def _regionChanging(self):
        """ internal: recalculate levels based on new position of adjusters """
        if not self.region_changed_enable: return
        bot, top = self.region.getRegion()
        bot = ( (bot -  63) / 64 ) # -1 to +1 over half-bar range
        top = ( (top - 191) / 64 ) # -1 to +1 over half-bar range
        bot = math.copysign( bot**2, bot ) # quadratic behaviour for sensitivity to small changes
        top = math.copysign( top**2, top )
        # These are the new values if adjuster is released now, rate of change depends on original separation
        span_prv = self.hi_prv - self.lo_prv # previous span of values
        hi_new = self.hi_prv + (span_prv + 2*self.rounding) * top # make sure that we can always
        lo_new = self.lo_prv + (span_prv + 2*self.rounding) * bot #   reach 2x the minimal step
        # Alternative model with speed depending on level magnitude:
        # mean_val = abs(self.lo_prv) + abs(self.hi_prv) / 2
        # hi_new = self.hi_prv + (mean_val + 2*self.rounding) * top # make sure that we can always
        # lo_new = self.lo_prv + (mean_val + 2*self.rounding) * bot #    reach 2x the minimal step

        if self.hi_lim is not None and hi_new > self.hi_lim: # limit maximum value
            # print('lim +')
            hi_new = self.hi_lim
            if lo_new > hi_new - span_prv: # avoid collapsing the span against top or bottom limits
                lo_new = hi_new - span_prv
        if self.lo_lim is not None and lo_new < self.lo_lim: # limit minimum value
            # print('lim -')
            lo_new = self.lo_lim
            if hi_new < lo_new + span_prv: # avoid collapsing the span against top or bottom limits
                hi_new = lo_new + span_prv
        if lo_new + self.rounding > hi_new: # do not allow less than one "rounding" unit of span
            # print('lim X')
            if   bot == 0: hi_new = lo_new + self.rounding
            elif top == 0: lo_new = hi_new - self.rounding
            else:
                lo_new = (lo_new + hi_new - self.rounding) / 2
                hi_new = lo_new + self.rounding
        lo_new = self.rounding * round( lo_new/self.rounding )
        hi_new = self.rounding * round( hi_new/self.rounding )
        # if hi_new == lo_new: hi_new = lo_new + self.rounding # hack solution if axis span still collapses
        self.values = (lo_new, hi_new)
        self._update_items()
        self.sigLevelsChanged.emit(self)



"""For starting up remote processes"""
import sys, pickle, os
import importlib

if __name__ == '__main__':
    if hasattr(os, 'setpgrp'):
        os.setpgrp()  ## prevents signals (notably keyboard interrupt) being forwarded from parent to this process
    if sys.version[0] == '3':
        #name, port, authkey, ppid, targetStr, path, pyside = pickle.load(sys.stdin.buffer)
        opts = pickle.load(sys.stdin.buffer)
    else:
        #name, port, authkey, ppid, targetStr, path, pyside = pickle.load(sys.stdin)
        opts = pickle.load(sys.stdin)
    #print "key:",  ' '.join([str(ord(x)) for x in authkey])
    path = opts.pop('path', None)
    if path is not None:
        if isinstance(path, str):
            # if string, just insert this into the path
            sys.path.insert(0, path)
        else:
            # if list, then replace the entire sys.path
            ## modify sys.path in place--no idea who already has a reference to the existing list.
            while len(sys.path) > 0:
                sys.path.pop()
            sys.path.extend(path)

    pyqtapis = opts.pop('pyqtapis', None)
    if pyqtapis is not None:
        try:
            from PyQt5 import sip
        except ImportError:
            import sip
            for k,v in pyqtapis.items():
                sip.setapi(k, v)
        
    qt_lib = opts.pop('qt_lib', None)
    if qt_lib is not None:
        globals()[qt_lib] = importlib.import_module(qt_lib)
    
    targetStr = opts.pop('targetStr')
    try:
        target = pickle.loads(targetStr)  ## unpickling the target should import everything we need
    except:
        print("Current sys.path:", sys.path)
        raise
    target(**opts)  ## Send all other options to the target function
    sys.exit(0)



# -*- coding: utf-8 -*-
"""
Simple logarithmic plotting test
"""

import initExample ## Add path to library (just for examples; you do not need this)


from pyqtgraph.Qt import QtGui, QtCore
import numpy as np
import pyqtgraph as pg

app = pg.mkQApp("Log Plot Example")

win = pg.GraphicsLayoutWidget(show=True, title="Basic plotting examples")
win.resize(1000,600)
win.setWindowTitle('pyqtgraph example: LogPlotTest')


p5 = win.addPlot(title="Scatter plot, axis labels, log scale")
x = np.random.normal(size=1000) * 1e-5
y = x*1000 + 0.005 * np.random.normal(size=1000)
y -= y.min()-1.0
mask = x > 1e-15
x = x[mask]
y = y[mask]
p5.plot(x, y, pen=None, symbol='t', symbolPen=None, symbolSize=10, symbolBrush=(100, 100, 255, 50))
p5.setLabel('left', "Y Axis", units='A')
p5.setLabel('bottom', "Y Axis", units='s')
p5.setLogMode(x=True, y=False)


if __name__ == '__main__':
    pg.exec()
